COST_USAGE_REPORT_DIMENSIONS = {
    'resource_name': 'ResourceName',
    'lease_type': 'OfferingType',
    'usage_family': 'UsageFamily',
    'instance_category': 'InstanceCategory',
    'usage_type': 'UsageType',
    'item_description': 'ItemDescription',
    'region': 'Region',
    'service_name': 'ServiceName',
    'date': 'StartDate',
    'category4': 'CostCenter',
    'vendor_account_name': 'AccountName',
    'vendor': 'PublicCloudName',
    'vendor_account_identifier': 'AccountId',
    'tag26': 'ResourcePlanId'
}

COST_USAGE_REPORT_METRICS = {
    'unblended_cost': 'Cost'
}
